/*----------------------------------------------------------------------------
	
	Designers Guide to the Cortex-M Family
	CMSIS RTOS Signal Example

*----------------------------------------------------------------------------*/
#include "stm32f10x.h"
#include <cmsis_os2.h>
#include "Board_LED.h"

void led_Thread1 (void  *argument);
void led_Thread2 (void  *argument);
void led_Thread3 (void  *argument);

osEventFlagsId_t EventFlag_LED;
osThreadId_t T_led_ID1;
osThreadId_t T_led_ID2;
osThreadId_t T_led_ID3;
/*----------------------------------------------------------------------------
 Simple delay loop
 *---------------------------------------------------------------------------*/

void delay (void){
unsigned int index;
const unsigned int count = 100000;
	
	for(index =0;index<count;index++){
		;
	}
}

/*----------------------------------------------------------------------------
  Flash LED 1 when signaled by the led_Thread2
 *---------------------------------------------------------------------------*/

static const osThreadAttr_t ThreadAttr_LED3 = {
	.name = "LED3",
	.priority = osPriorityAboveNormal,
};

__NO_RETURN void led_Thread3 (void *argument) {
	
	for (;;) {
		osEventFlagsWait (EventFlag_LED,0x01,osFlagsWaitAny,osWaitForever);
		LED_On(3);                          
		osEventFlagsWait (EventFlag_LED,0x01,osFlagsWaitAny,osWaitForever);	
		LED_Off(3);
	}
}
/*----------------------------------------------------------------------------
  Flash LED 1 when signaled by the led_Thread2
 *---------------------------------------------------------------------------*/

static const osThreadAttr_t ThreadAttr_LED1 = {
	.name = "LED1",
	.priority = osPriorityAboveNormal,
};

__NO_RETURN void led_Thread1 (void *argument) {
	
	for (;;) {
		osEventFlagsWait (EventFlag_LED,0x01,osFlagsWaitAny,osWaitForever);
		LED_On(1);                          
		osEventFlagsWait (EventFlag_LED,0x01,osFlagsWaitAny,osWaitForever);	
		LED_Off(1);
	}
}
/*----------------------------------------------------------------------------
  Flash LED two and synchronise the flashing of LED 1 by setting a signal flag
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_LED2 = {
	.name = "LED2",
	.priority = osPriorityNormal,
};

__NO_RETURN void led_Thread2 (void *argument) {
	
	for (;;) 	{
		osEventFlagsSet	(EventFlag_LED,0x01);
		LED_On(2);	
		osDelay(10);
		LED_Off(2);
		osEventFlagsSet	(EventFlag_LED,0x01);
		osDelay(10);
	}
}

/*----------------------------------------------------------------------------
  Start the threads
 *---------------------------------------------------------------------------*/

static const osThreadAttr_t ThreadAttr_main = {
	.name = "main",
	.priority = osPriorityNormal,
};

static const osEventFlagsAttr_t EventFlagAttr_LED = {
	.name = "LED_Events",
};
void app_main (void *argument) 
{
	
	LED_Initialize();
	EventFlag_LED = osEventFlagsNew(&EventFlagAttr_LED);
	T_led_ID1 = osThreadNew(led_Thread1, NULL, &ThreadAttr_LED1);
	T_led_ID2 = osThreadNew(led_Thread2, NULL,&ThreadAttr_LED2);	
	T_led_ID3 = osThreadNew(led_Thread3, NULL,&ThreadAttr_LED3);
	osDelay(osWaitForever);
  
  while(1);
}

int main (void) {

  SystemCoreClockUpdate();
  osKernelInitialize();                 						// Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL,&ThreadAttr_main);    	// Create application main thread
  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                    						// Start thread execution
  }

  while(1);
}
