/*----------------------------------------------------------------------------
	
	Designers Guide to the Cortex-M Family
	Semaphore Example
*----------------------------------------------------------------------------*/

#include "stm32f10x.h"
#include "cmsis_os2.h"
#include "Board_LED.h"

void led_Thread1 (void *argument);
void led_Thread2 (void *argument);

osThreadId_t T_led1, T_led2;

osSemaphoreId_t sem1;

/*----------------------------------------------------------------------------
  Wait to acquire a semaphore token from sem1 then flash LED 1
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_LED1 = {
	.name = "LED1",
	.priority = osPriorityAboveNormal,
};

__NO_RETURN void led_Thread1 (void  *argument) {
	
	for (;;) {
		osSemaphoreAcquire(sem1, osWaitForever);
		LED_On(1);                          
		osSemaphoreAcquire(sem1, osWaitForever);
		LED_Off(1);
	}
}
/*----------------------------------------------------------------------------
  Flash LED 2 and 'release' a semaphore token to sem1
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_LED2 = {
	.name = "LED2",
	.priority = osPriorityNormal,
};

__NO_RETURN void led_Thread2 (void *argument) {
	
	for (;;) {
		osSemaphoreRelease(sem1);
		LED_On(2);		
		osDelay(500);
		osSemaphoreRelease(sem1);
		LED_Off(2);
		osDelay(500);
	}
}

/*----------------------------------------------------------------------------
  Initilise the LED's, Create the semaphore and start the threads
 *---------------------------------------------------------------------------*/
static const osSemaphoreAttr_t semAttr_SEM1 = {
	.name = "SEM1",
};

static const osThreadAttr_t ThreadAttr_main = {
	.name = "main",
	.priority = osPriorityNormal,
};

void app_main (void *argument) {
	
	sem1 = osSemaphoreNew(5, 0, &semAttr_SEM1 );	
	T_led1 = osThreadNew(led_Thread1, NULL, &ThreadAttr_LED1);
	T_led2 = osThreadNew(led_Thread2, NULL, &ThreadAttr_LED2);
}


int main (void) {

  SystemCoreClockUpdate();
  LED_Initialize ();
  osKernelInitialize();                							// Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL, &ThreadAttr_main);    // Create application main thread
  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                    						// Start thread execution
  }
  while(1);
}
