/*----------------------------------------------------------------------------
	
	Designers Guide to the Cortex-M Family
	CMSIS RTOS Multiplex Example

*----------------------------------------------------------------------------*/
#include "stm32f10x.h"
#include "cmsis_os2.h"

#include "Board_LED.h"

void  multiplex_Thread (void *argument);

osSemaphoreId_t semMultiplex;
osThreadId_t T_mux1, T_mux2, T_mux3, T_mux4, T_mux5, T_mux6;

/*******************************************************************
Create a thread and use the Semaphore to control the number of threads 
running simultaneously
********************************************************************/

void multiplex_Thread (void *argument) 
{
	for (;;) 
	{
    osSemaphoreAcquire(semMultiplex,osWaitForever);
		LED_On((uint32_t)argument);
		osDelay(100);
		LED_Off((uint32_t)argument);
		osDelay(100);
		osSemaphoreRelease(semMultiplex);
	}
}

/*----------------------------------------------------------------------------
 Create the Semaphore and launch the threads
 *---------------------------------------------------------------------------*/
static const osSemaphoreAttr_t semAttr_MULTIPLEX = {
	.name = "SEM1",
};


int main (void) 
{
	osKernelInitialize ();                    // initialize CMSIS-RTOS
	
	LED_Initialize();
	semMultiplex = osSemaphoreNew(5, 2, &semAttr_MULTIPLEX );		
	T_mux1 = osThreadNew(multiplex_Thread,(void *)0,NULL);
	T_mux2 = osThreadNew(multiplex_Thread,(void *)1,NULL);                  
	T_mux3 = osThreadNew(multiplex_Thread,(void*)2,NULL);
	T_mux4 = osThreadNew(multiplex_Thread,(void*)3,NULL); 
	T_mux5 = osThreadNew(multiplex_Thread,(void*)4,NULL);
	T_mux6 = osThreadNew(multiplex_Thread,(void*)5,NULL);
	
	osKernelStart ();                         // start thread execution 
}
