/*----------------------------------------------------------------------------
	
	Designers Guide to the Cortex-M Family
	CMSIS RTOS Semaphore Rendezvous Example

*----------------------------------------------------------------------------*/
#include "stm32f10x.h"
#include <cmsis_os2.h>
#include "Board_LED.h"

void led_Thread1 (void  *argument);
void led_Thread2 (void  *argument);

osThreadId_t T_led_Thread1, T_led_Thread2;
osSemaphoreId_t semArrived1,semArrived2;	

/*----------------------------------------------------------------------------
  Synchronise the LED's using a semaphore rendezvous
 *---------------------------------------------------------------------------*/
 static const osThreadAttr_t ThreadAttr_LED1 = {
	.name = "LED1",
};
 
__NO_RETURN void led_Thread1 (void *argument) {
	
	for (;;){
		LED_Off(1);
		osDelay(100);
//		osSemaphoreRelease(semArrived2);									//The semaphores ensure both tasks arrive here
//		osSemaphoreAcquire(semArrived1,osWaitForever);		//before continuing
		LED_On(1);
		osDelay(500);
//		osSemaphoreRelease(semArrived2);									//The semaphores ensure both tasks arrive here
//		osSemaphoreAcquire(semArrived1,osWaitForever);		//before continuing
  }
}

/*----------------------------------------------------------------------------
   Synchronise the LED's using a semaphore rendezvous
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_LED2 = {
	.name = "LED2",
};

__NO_RETURN void led_Thread2 (void *argument) {
	
	for (;;) {
		LED_Off(2); 
		osDelay(500);
//		osSemaphoreRelease(semArrived1);									//The semaphores ensure both tasks arrive here
//		osSemaphoreAcquire(semArrived2,osWaitForever);		//before continuing
		LED_On(2);
		osDelay(100);
//		osSemaphoreRelease(semArrived1);									//The semaphores ensure both tasks arrive here
//		osSemaphoreAcquire(semArrived2,osWaitForever);		//before continuing
	}
}

/*----------------------------------------------------------------------------
 Create the semaphores and start the threads
 *---------------------------------------------------------------------------*/

static const osSemaphoreAttr_t semAttr_SEM1 = {
	.name = "SEM1",
};

static const osSemaphoreAttr_t semAttr_SEM2 = {
	.name = "SEM2",
};

static const osThreadAttr_t ThreadAttr_main = {
	.name = "main",
};

void app_main (void *argument) 
{
	
	semArrived1 = osSemaphoreNew(1,0,&semAttr_SEM1);				
	semArrived2 = osSemaphoreNew(1,0,&semAttr_SEM2);		
	T_led_Thread1 =	osThreadNew(led_Thread1, NULL, &ThreadAttr_LED1);
	T_led_Thread2 =	osThreadNew(led_Thread2, NULL, &ThreadAttr_LED2);
}

int main (void) {

	SystemCoreClockUpdate();
  LED_Initialize ();
  osKernelInitialize();                 						// Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL, &ThreadAttr_main);    // Create application main thread
  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                    						// Start thread execution
  }

  while(1);
}
