/*----------------------------------------------------------------------------
	
	Designers Guide to the Cortex-M Family
	Semaphore Barrier Example

*----------------------------------------------------------------------------*/

#include "cmsis_os2.h"
#include "stm32f10x.h"                   
#include "Board_LED.h"

void threadBaseCode (void *argument) ;

osThreadId_t tsk_0,tsk_1,tsk_2,tsk_3,tsk_4;			
osSemaphoreId_t Turnstile_In,Turnstile_Out,Mutex;	

static const osSemaphoreAttr_t semAttr_SEM_In = {
	.name = "Turnstile_In",
};
static const osSemaphoreAttr_t semAttr_SEM_Out = {
	.name = "Turnstile_Out",
};
static const osSemaphoreAttr_t semAttr_Mutex = {
	.name = "Mutex",
};
static const osThreadAttr_t ThreadAttr_main = {
	.name = "main",
	.priority = osPriorityNormal,
};
static const osThreadAttr_t ThreadAttr_tsk_0 = {
	.name = "Task_0",
	.priority = osPriorityNormal,
};
static const osThreadAttr_t ThreadAttr_tsk_1 = {
	.name = "Task_1",
	.priority = osPriorityNormal,
};
static const osThreadAttr_t ThreadAttr_tsk_2 = {
	.name = "Task_2",
	.priority = osPriorityNormal,
};
static const osThreadAttr_t ThreadAttr_tsk_3 = {
	.name = "Task_3",
	.priority = osPriorityNormal,
};
static const osThreadAttr_t ThreadAttr_tsk_4 = {
	.name = "Task_4",
	.priority = osPriorityNormal,
};

unsigned int count = 0;
unsigned int param[5] = { 0x1,0x2,0x4,0x8,0x10};

__NO_RETURN void threadBaseCode  (void *argument) 			
{
uint32_t LED_data;
	
	LED_data = (uint32_t) argument;
	osDelay(10*LED_data);
	while(1)					
	{
/*
//-----------------------------------Entry Turnstile-------------------------------							
		osSemaphoreAcquire(Mutex,osWaitForever);			 							//Allow one task at a time to access the first turnstile
		count = count+1;					 																	// Increment count
		if( count == 5) 					 																	//When last section of code reaches this point run his code
		{
			osSemaphoreAcquire (Turnstile_Out,osWaitForever);	 				//Lock the second turnstile
			osSemaphoreRelease(Turnstile_In);				 									//Unlock the first turnstile
		}			
		osSemaphoreRelease(Mutex);																	//Allow other tasks to access the turnstile
		osSemaphoreAcquire(Turnstile_In,osWaitForever);							//Turnstile Gate			
		osSemaphoreRelease(Turnstile_In);
//-----------------------------------Entry Turnstile-----------------------------------
*/
		LED_On(LED_data);
		osDelay(20);				   							//Critical Section
		LED_Off(LED_data);
		osDelay(20);	
/*
//----------------------------------Exit Turnstile------------------------------------
		osSemaphoreAcquire(Mutex,osWaitForever);			 							//Allow one task at a time to access the turnstile
		count = count - 1;
		if(count ==0)						 																		//When last section of code reaches this point run his code
		{
			osSemaphoreAcquire(Turnstile_In,osWaitForever);		 				//Lock the second turnstile
			osSemaphoreRelease(Turnstile_Out);			 									//Unlock the first turnstile
		}
		osSemaphoreRelease(Mutex);					 												//Allow other tasks to access the turnstile
		osSemaphoreAcquire(Turnstile_Out,osWaitForever);		 				//Turnstile Gate
		osSemaphoreRelease(Turnstile_Out);
//---------------------------------Exit Turnstile-------------------------------------		
*/
	}
}	


/*----------------------------------------------------------------------------
 *        Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/

void app_main (void *argument) 
{
	
	Turnstile_In =	osSemaphoreNew(5, 0,&semAttr_SEM_In);
	Turnstile_Out = osSemaphoreNew(5, 1,&semAttr_SEM_Out);
	Mutex =	osSemaphoreNew(1, 1,&semAttr_Mutex);	
	tsk_0 = osThreadNew(threadBaseCode,(uint32_t *)0,&ThreadAttr_tsk_0);  
	tsk_1 = osThreadNew(threadBaseCode,(uint32_t *)1,&ThreadAttr_tsk_1); 
	tsk_2 = osThreadNew(threadBaseCode,(uint32_t *)2,&ThreadAttr_tsk_2);
	tsk_3 = osThreadNew(threadBaseCode,(uint32_t *)3,&ThreadAttr_tsk_3);
	tsk_4 = osThreadNew(threadBaseCode,(uint32_t *)4,&ThreadAttr_tsk_4);

}

int main (void) {

  SystemCoreClockUpdate();
  LED_Initialize ();
  osKernelInitialize();                							// Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL, &ThreadAttr_main);    // Create application main thread
  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                    						// Start thread execution
  }
  while(1);
}

/*----------------------------------------------------------------------------
 * end of file
 *---------------------------------------------------------------------------*/

