/*----------------------------------------------------------------------------
	
	Designers Guide to the Cortex-M Family
	CMSIS RTOS Message Queue Example

*----------------------------------------------------------------------------*/
#include "STM32F10x.h"
#include "cmsis_os2.h"
#include "Board_LED.h"

void led_Thread1 (void  *argument);																
void led_Thread2 (void  *argument);

osThreadId_t T_led_Thread1,T_led_Thread2, T_main; 
osMessageQueueId_t Q_LED;

/*----------------------------------------------------------------------------
  Task 2 'ledOff': switches the LED off
 *---------------------------------------------------------------------------*/
 static const osThreadAttr_t ThreadAttr_LED1 = {
	.name = "LED1",
};
 
__NO_RETURN void led_Thread1 (void *argument) {
uint32_t dataOut;
uint8_t priority;
	
	for (;;) {
		osDelay(100);
		osMessageQueueGet(Q_LED,&dataOut,&priority,osWaitForever);					//wait for a message to arrive
		LED_On(dataOut);																										// write the data to the LED's
		if(priority == osPriorityAboveNormal){
			LED_On(7);
		}
		osDelay(100);
		LED_Off(dataOut);
		if(priority == osPriorityAboveNormal){
			LED_Off(7);
		}
		
	}
}
/*----------------------------------------------------------------------------
  Task 1 'ledOn': switches the LED on
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_LED2 = {
	.name = "LED2",
};

__NO_RETURN void led_Thread2 (void *argument) {
uint32_t dataIn = 1;

	for (;;) {
		osMessageQueuePut(Q_LED,&dataIn,osPriorityNormal,osWaitForever);             	//Place a value in the message queue
		osDelay(10);																																	//Wait 100 ms
		dataIn = 0x02;
		osMessageQueuePut(Q_LED,&dataIn,osPriorityAboveNormal,osWaitForever);					//Repeat for other bit patterns
		osDelay(100);
		dataIn = 0x03;
		osMessageQueuePut(Q_LED,&dataIn,osPriorityNormal,osWaitForever);
		osDelay(100);
		dataIn = 0x04;
		osMessageQueuePut(Q_LED,&dataIn,osPriorityNormal,osWaitForever);
		osDelay(100);
		dataIn = 0x01;
		osDelay(100);
	}
}

/*----------------------------------------------------------------------------
  Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/
static const osMessageQueueAttr_t queueAttr_Q_LED = {
		.name = "LED_QUEUE",
};

void app_main (void *argument) 
{
		
	LED_Initialize ();
	Q_LED = osMessageQueueNew(16,4,&queueAttr_Q_LED );					//create the message queue
	T_led_Thread1 = osThreadNew(led_Thread1, NULL, &ThreadAttr_LED1);
	T_led_Thread2 =	osThreadNew(led_Thread2, NULL, &ThreadAttr_LED2);
}

int main (void) {

  SystemCoreClockUpdate();
  LED_Initialize ();
  osKernelInitialize();                 			// Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL, NULL);    			// Create application main thread
  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                    			// Start thread execution
  }

  while(1);
}
