/*----------------------------------------------------------------------------
	
	Designers Guide to the Cortex-M Family
	CMSIS RTOS Message Queue Example

*----------------------------------------------------------------------------*/

#include <cmsis_os2.h>
#include "Board_LED.h"
#include "stm32f10x.h"

typedef struct {
    uint8_t canData[8];
} message_t;
 
osMemoryPoolId_t mpool;
osMessageQueueId_t queue;

/*----------------------------------------------------------------------------
 Producer thread for mailbox
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_producer = {
	.name = "producer",
};

__NO_RETURN void producer_thread (void  *args) {
message_t *message;																									//define pointer for memory pool buffer
uint8_t index,testData = 0x55;

	while (1){
			if(testData == 0xAA){
				testData = 0x55;
			}
			else{
				testData = 0xAA;
			}
      message = (message_t*)osMemoryPoolAlloc(mpool,osWaitForever);			//Allocate a memory pool buffer
			for(index =0;index<8;index++){
        message->canData[index] = testData;															//populate buffer with data
			}
       osMessageQueuePut(queue, &message,NULL, osWaitForever);					//Post pointer to memory pool buffer
       osDelay(100);
    }
}
/*----------------------------------------------------------------------------
  Consumer thread for mailbox
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_consumer = {
	.name = "consumer",
};

__NO_RETURN void consumer_thread (void  *args){
message_t *message;
	
	while (1) {
		osMessageQueueGet(queue,&message,NULL,osWaitForever);	
		LED_SetOut((uint32_t)message->canData[0]);													//write the data to the LED's
		osMemoryPoolFree(mpool, message);																		//Release the buffer
	}
}

 /*----------------------------------------------------------------------------
 Start the RTOS
 *---------------------------------------------------------------------------*/
static const osMemoryPoolAttr_t memorypoolAttr_mpool ={
	.name = "memory_pool",
};

static const osThreadAttr_t ThreadAttr_main = {
	.name = "main",
};

void app_main (void *argument) {
			
    mpool = osMemoryPoolNew(16, sizeof(message_t),&memorypoolAttr_mpool );
    queue = osMessageQueueNew(16,4, NULL);
    osThreadNew(producer_thread, NULL, &ThreadAttr_producer);
    osThreadNew(consumer_thread, NULL, &ThreadAttr_consumer);
}

int main (void) {

  SystemCoreClockUpdate();
  LED_Initialize ();
  osKernelInitialize();                 									// Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL, &ThreadAttr_main);    			// Create application main thread
  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                    									// Start thread execution
  }

  while(1);
}
