/*----------------------------------------------------------------------------
	
	Designers Guide to the Cortex-M Family
	CMSIS RTOS Threads Example

*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Include the microcontroller header for register defenitions and CMSIS core functions
	Include the CMSIS RTOS header for the RTOS API
 *---------------------------------------------------------------------------*/

#include "STM32F10x.h"
#include <cmsis_os2.h>
#include "Board_LED.h"

#define LED1_ON 1
#define LED1_OFF 0

osThreadId_t main_ID,led_ID1,led_ID2,worker_ID1,worker_ID2;

/*----------------------------------------------------------------------------
  Simple delay routine
 *---------------------------------------------------------------------------*/
void delay (unsigned int count)
{
unsigned int index;

	for(index =0;index<count;index++){
		;
	}
}

/*----------------------------------------------------------------------------
 Worker Thread - Created as joinable
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_worker = {
	.name = "Worker",
	.attr_bits = osThreadJoinable,
	.priority = osPriorityAboveNormal,
};

void worker_thread (void *argument) {

		if((uint32_t)argument == LED1_ON) {
			LED_On(1);
		}
		else if ((uint32_t)argument == LED1_OFF){	
			LED_Off(1);
		}
		delay(500);
		osThreadExit();
}

/*----------------------------------------------------------------------------
 LED Flasher thread
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_LED1 = {
	.name = "LED1",
	.priority = osPriorityAboveNormal,	
};

__NO_RETURN void led_thread1 (void *argument) {
  
	for (;;) {
		worker_ID1   = osThreadNew(worker_thread,(void *) LED1_ON, &ThreadAttr_worker);         //Create worker thread to do some calculations               
		LED_On(2);																																							// Do something else in parallel
		osThreadJoin(worker_ID1);																																// Wait for the worker thread to finish so you can use its results		
		worker_ID2   = osThreadNew(worker_thread,(void *) LED1_OFF, &ThreadAttr_worker);
		LED_Off(2);
		osThreadJoin(worker_ID2);
	}
}

/*----------------------------------------------------------------------------
 Initilise the LED's and start the RTOS
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_app_main = {
	.name = "app_main",	
};

void app_main (void *argument) {

	LED_Initialize ();
	led_ID1 = osThreadNew(led_thread1, NULL, &ThreadAttr_LED1);
}

int main (void) {

  SystemCoreClockUpdate();
 	LED_Initialize ();
  osKernelInitialize();                 				// Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL, &ThreadAttr_app_main);    				// Create application main thread
  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                    				// Start thread execution
  }

  while(1);
}
 
