/*----------------------------------------------------------------------------
	
	Designers Guide to the Cortex-M Family
	CMSIS RTOS Signal Example

*----------------------------------------------------------------------------*/
#include "stm32f10x.h"
#include <cmsis_os2.h>
#include "Board_LED.h"

void led_Thread1 (void  *argument);
void led_Thread2 (void  *argument);



osThreadId_t T_led_ID1;
osThreadId_t T_led_ID2;

void delay (void){
unsigned int index;
const unsigned int count = 100000;
	
	for(index =0;index<count;index++){
		;
	}
}
/*----------------------------------------------------------------------------
  Flash LED 1 when signaled by the led_Thread2
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_LED1 = {
	.name = "LED1",
	.priority = osPriorityAboveNormal,
};

__NO_RETURN void led_Thread1 (void *argument) {
	
	for (;;) {
		osThreadFlagsWait (0x01,osFlagsWaitAny,osWaitForever);
		LED_On(1);                          
		osThreadFlagsWait (0x01,osFlagsWaitAny,osWaitForever);	
		LED_Off(1);
	}
}
/*----------------------------------------------------------------------------
  Flash LED two and synchronise the flashing of LED 1 by setting a signal flag
 *---------------------------------------------------------------------------*/
static const osThreadAttr_t ThreadAttr_LED2 = {
	.name = "LED2",
	.priority = osPriorityNormal,
};

__NO_RETURN void led_Thread2 (void *argument) {
	
	for (;;) {
		LED_On(2);		
		osThreadFlagsSet	(T_led_ID1,0x01);
		osDelay(100);
		LED_Off(2);
		osThreadFlagsSet	(T_led_ID1,0x01);
		osDelay(100);
	}
}




/*----------------------------------------------------------------------------
  Start the threads
 *---------------------------------------------------------------------------*/
void app_main (void *argument) {
	
	LED_Initialize();
	T_led_ID1 = osThreadNew(led_Thread1, NULL, &ThreadAttr_LED1);
	T_led_ID2 = osThreadNew(led_Thread2, NULL,&ThreadAttr_LED2);	
	osDelay(osWaitForever);
  while(1);
}

int main (void) {

  SystemCoreClockUpdate();
  osKernelInitialize();                 				// Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL, NULL);    				// Create application main thread
  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                    				// Start thread execution
  }

  while(1);
}
